@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2015-2017 Montel Laurent <montel@kde.org>
+  Copyright (c) 2015-2018 Montel Laurent <montel@kde.org>
 
   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU Library General Public License as published by
@@ -21,16 +21,15 @@
 #include "kdebugsettingutiltest.h"
 #include "../src/kdebugsettingsutil.h"
 #include <qtest.h>
+#include <QDebug>
 
 KDebugSettingUtilTest::KDebugSettingUtilTest(QObject *parent)
     : QObject(parent)
 {
-
 }
 
 KDebugSettingUtilTest::~KDebugSettingUtilTest()
 {
-
 }
 
 void KDebugSettingUtilTest::shouldParseKdeLoggingLine_data()
@@ -38,16 +37,37 @@ void KDebugSettingUtilTest::shouldParseKdeLoggingLine_data()
     QTest::addColumn<QString>("input");
     QTest::addColumn<QString>("description");
     QTest::addColumn<QString>("logname");
+    QTest::addColumn<QString>("defaultCategory");
     QTest::addColumn<bool>("valid");
-    QTest::newRow("empty") <<  QString() << QString() << QString() << false;
-    QTest::newRow("validLine") << QStringLiteral("log linux") << QStringLiteral("linux") << QStringLiteral("log") << true;
-    QTest::newRow("validLinewithspace") << QStringLiteral(" log linux  ") << QStringLiteral("linux") << QStringLiteral("log") << true;
-    QTest::newRow("comment") << QStringLiteral("#log linux  ") << QString() << QString() << false;
-    QTest::newRow("commentWithSpace") << QStringLiteral("   #log linux  ") << QString() << QString() << false;
-    QTest::newRow("badline") << QStringLiteral("log") << QString() << QString() << false;
-    QTest::newRow("comment-2") << QStringLiteral("#log linux") << QString() << QString() << false;
+    QTest::newRow("empty") <<  QString() << QString() << QString() << QString() << false;
 
-    QTest::newRow("linewithcomment") << QStringLiteral("log linux#comment about linux") << QStringLiteral("linux") << QStringLiteral("log") << true;
+    QTest::newRow("validLine") << QStringLiteral("log linux") << QStringLiteral("linux") << QStringLiteral("log") << QString() << true;
+    QTest::newRow("validLinewithspace") << QStringLiteral(" log linux  ") << QStringLiteral("linux") << QStringLiteral("log") << QString() << true;
+    QTest::newRow("comment") << QStringLiteral("#log linux  ") << QString() << QString() << QString() << false;
+    QTest::newRow("commentWithSpace") << QStringLiteral("   #log linux  ") << QString() << QString() << QString() << false;
+    QTest::newRow("badline") << QStringLiteral("log") << QString() << QString() << QString() << false;
+    QTest::newRow("comment-2") << QStringLiteral("#log linux") << QString() << QString() << QString() << false;
+    QTest::newRow("validLineWithParentheses") << QStringLiteral("log linux (foo)") << QStringLiteral("linux (foo)") << QStringLiteral("log") << QString() << true;
+
+    QTest::newRow("validLineWithParenthesesAndCategories") << QStringLiteral("log linux (foo) [WARNING]") << QStringLiteral("linux (foo)") << QStringLiteral("log") <<  QStringLiteral("WARNING") << true;
+
+    QTest::newRow("validLineCategoriesWarning") << QStringLiteral("log linux [WARNING]") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("WARNING") << true;
+    QTest::newRow("validLineCategoriesWarning2") << QStringLiteral("log linux [WARNING]    ") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("WARNING") << true;
+    QTest::newRow("validLineCategoriesWarning3") << QStringLiteral("log linux      [WARNING]    ") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("WARNING") << true;
+    QTest::newRow("validLineBadCategories") << QStringLiteral("log linux      [FOO]    ") << QStringLiteral("linux [FOO]") << QStringLiteral("log") << QString() << true;
+    QTest::newRow("linewithcomment") << QStringLiteral("log linux#comment about linux") << QStringLiteral("linux") << QStringLiteral("log") << QString() << true;
+
+    QTest::newRow("validLineCategoriesInfo") << QStringLiteral("log linux [INFO]") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("INFO") << true;
+    QTest::newRow("validLineCategoriesInfo2") << QStringLiteral("log linux [INFO]    ") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("INFO") << true;
+    QTest::newRow("validLineCategoriesInfo3") << QStringLiteral("log linux      [INFO]    ") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("INFO") << true;
+
+    QTest::newRow("validLineCategoriesDEBUG") << QStringLiteral("log linux [DEBUG]") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("DEBUG") << true;
+    QTest::newRow("validLineCategoriesDEBUG2") << QStringLiteral("log linux [DEBUG]    ") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("DEBUG") << true;
+    QTest::newRow("validLineCategoriesDEBUG3") << QStringLiteral("log linux      [DEBUG]    ") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("DEBUG") << true;
+
+    QTest::newRow("validLineCategoriesCRITICAL") << QStringLiteral("log linux [CRITICAL]") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("CRITICAL") << true;
+    QTest::newRow("validLineCategoriesCRITICAL2") << QStringLiteral("log linux [CRITICAL]    ") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("CRITICAL") << true;
+    QTest::newRow("validLineCategoriesCRITICAL3") << QStringLiteral("log linux      [CRITICAL]    ") << QStringLiteral("linux") << QStringLiteral("log") << QStringLiteral("CRITICAL") << true;
 }
 
 void KDebugSettingUtilTest::shouldParseKdeLoggingLine()
@@ -55,11 +75,17 @@ void KDebugSettingUtilTest::shouldParseKdeLoggingLine()
     QFETCH(QString, input);
     QFETCH(QString, description);
     QFETCH(QString, logname);
+    QFETCH(QString, defaultCategory);
     QFETCH(bool, valid);
     KdeLoggingCategory result;
     result.description = description;
     result.logName = logname;
+    result.defaultCategory = defaultCategory;
     const KdeLoggingCategory cat = KDebugSettingsUtil::parseLineKdeLoggingCategory(input);
+    if (cat != result) {
+        qDebug() << "Generated category " << cat;
+        qDebug() << "Expected category: "<<result;
+    }
     QCOMPARE(cat, result);
     QCOMPARE(cat.isValid(), valid);
 }
@@ -146,4 +172,23 @@ void KDebugSettingUtilTest::shouldReadRenameCategories()
     QCOMPARE(lst.count(), numberofrenamecategories);
 }
 
+void KDebugSettingUtilTest::shouldConvertCategoryTypeFromString_data()
+{
+    QTest::addColumn<QString>("input");
+    QTest::addColumn<LoggingCategory::LoggingType>("loggingType");
+    QTest::newRow("empty") << QString() << LoggingCategory::LoggingType::Info;
+    QTest::newRow("WARNING") << QStringLiteral("WARNING") << LoggingCategory::LoggingType::Warning;
+    QTest::newRow("Info") << QStringLiteral("INFO") << LoggingCategory::LoggingType::Info;
+    QTest::newRow("Debug") << QStringLiteral("DEBUG") << LoggingCategory::LoggingType::Debug;
+    QTest::newRow("Critical") << QStringLiteral("CRITICAL") << LoggingCategory::LoggingType::Critical;
+    QTest::newRow("unknow") << QStringLiteral("foo") << LoggingCategory::LoggingType::Info;
+}
+
+void KDebugSettingUtilTest::shouldConvertCategoryTypeFromString()
+{
+    QFETCH(QString, input);
+    QFETCH(LoggingCategory::LoggingType, loggingType);
+    QCOMPARE(KDebugSettingsUtil::convertCategoryTypeFromString(input), loggingType);
+}
+
 QTEST_GUILESS_MAIN(KDebugSettingUtilTest)
